@@ -1,11 +1,11 @@
 from RupineHeroku.rupine_db import herokuDbAccess
 from psycopg2 import sql
-#from datetime import datetime
-#import pytz
+from datetime import datetime
+import pytz
 
 def postTaxTransaction(connection, schema, data):
 
-    query = sql.SQL("INSERT INTO {}.tax_transaction (tx_no,chain_id,chain,public_address,timestamp,block_number,transaction_hash,category,token,amount,usd_value,eur_value) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
+    query = sql.SQL("INSERT INTO {}.tax_transaction (tx_no,chain_id,chain,public_address,timestamp,block_number,transaction_hash,category,token,amount,usd_value,eur_value,fee_token,fee_amount,fee_usd_value,fee_eur_value) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
     params = (
         data['tx_no'],
         data['chain_id'],
@@ -18,7 +18,11 @@ def postTaxTransaction(connection, schema, data):
         data['token'],
         data['amount'],
         data['usd_value'],
-        data['eur_value'])
+        data['eur_value'],
+        data['fee_token'],
+        data['fee_amount'],
+        data['fee_usd_value'],
+        data['fee_eur_value'])
     result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
     return result
 
@@ -57,12 +61,12 @@ def getTaxTransactionRewards(connection, schema, addresses:list=None, chain=None
         conditions = conditions + " AND t.amount < 0"
 
     query = sql.SQL("SELECT basis.* FROM ( \
-                        SELECT t.id,t.chain_id,t.chain,t.public_address,t.timestamp,t.block_number,t.transaction_hash,t.category,t.token,t.amount,t.usd_value,t.eur_value, \
+                        SELECT t.id,t.chain_id,t.chain,t.public_address,t.timestamp,t.block_number,t.transaction_hash,t.category,t.token,t.amount,t.usd_value,t.eur_value,t.fee_usd_value,t.fee_eur_value, \
                         CASE WHEN c.sort_no IS NULL THEN '999' ELSE c.sort_no END AS sort_no \
                         FROM {0}.tax_transaction t LEFT JOIN {0}.tax_category c ON t.category = c.category \
                         WHERE 1=1 AND t.timestamp >= %%s %s \
                             UNION ALL	\
-                        SELECT t.id,t.chain_id,t.chain,t.public_address,t.timestamp,NULL AS block_number,NULL AS transaction_hash,t.category,t.token,t.amount,t.usd_value,t.eur_value, \
+                        SELECT t.id,t.chain_id,t.chain,t.public_address,t.timestamp,NULL AS block_number,NULL AS transaction_hash,t.category,t.token,t.amount,t.usd_value,t.eur_value,NULL AS fee_usd_value,NULL AS fee_eur_value, \
                         CASE WHEN c.sort_no IS NULL THEN '999' ELSE c.sort_no END AS sort_no \
                         FROM {0}.tax_reward t LEFT JOIN {0}.tax_category c ON t.category = c.category \
                         WHERE 1=1 AND t.timestamp >= %%s %s \
@@ -106,7 +110,7 @@ def getTaxTokens(connection, schema, addresses:list=None, chain=None, chain_id=N
     result = herokuDbAccess.fetchDataInDatabase(query, [*params,*params], connection)  
     return result
 
-def getLatestWarehouseDate(connection, schema, token:str, account:str, chain=None, chain_id=None):
+def getLatestWarehouseTxDate(connection, schema, token:str, account:str, chain=None, chain_id=None):
     conditions = ""
     params = []
     if chain is None and chain_id is None:
@@ -135,6 +139,27 @@ def getLatestWarehouseDate(connection, schema, token:str, account:str, chain=Non
     return result[0]
     #return datetime(d.year,d.month,d.day, 0, 0, 0, tzinfo=pytz.timezone('UTC'))
 
+def getLatestWarehouseDate(connection, schema, token:str, account:str, chain=None, chain_id=None):
+    conditions = ""
+    params = []
+    if chain is None and chain_id is None:
+        return []
+    elif chain is not None:
+        conditions = conditions + " AND chain = %s"
+        params.append(chain)
+    elif chain_id is not None:
+        conditions = conditions + " AND chain_id = %s"
+        params.append(chain_id)
+
+    query = sql.SQL("SELECT MAX(day) as latest_day FROM {0}.tax_warehouse WHERE 1=1 AND account = %%s AND token = %%s %s" % (conditions)).format(sql.Identifier(schema))
+
+    result = herokuDbAccess.fetchDataInDatabase(query, [account,token,*params], connection)  
+    if result[0][0] is None:
+        return datetime(1970, 1, 1, 0, 0, 0, tzinfo=pytz.timezone('UTC'))
+        
+    d = result[0][0]
+    return datetime(d.year,d.month,d.day, 0, 0, 0, tzinfo=pytz.timezone('UTC'))
+
 
 def postTaxReward(connection, schema, data):
     query = sql.SQL("INSERT INTO {}.tax_reward (tx_no,chain_id,chain,public_address,timestamp,category,token,amount,usd_value,eur_value) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))   
@@ -155,7 +180,7 @@ def postTaxReward(connection, schema, data):
     return result
 
 def postTaxWarehouse(connection, schema, data):
-    query = sql.SQL("INSERT INTO {}.tax_warehouse (chain_id,chain,account,day,token,amount,amount_usd,amount_eur,amount_in,amount_in_usd,amount_in_eur,amount_out,amount_out_usd,amount_out_eur,tax_amount_usd,tax_amount_eur,rest_tx_no,rest_amount) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))   
+    query = sql.SQL("INSERT INTO {}.tax_warehouse (chain_id,chain,account,day,token,amount,amount_usd,amount_eur,amount_in,amount_in_usd,amount_in_eur,amount_out,amount_out_usd,amount_out_eur,profit_amount_usd,profit_amount_eur,fee_amount_usd,fee_amount_eur,rest_tx_no,rest_amount) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))   
     params = (
         data['chain_id'],
         data['chain'],
@@ -171,8 +196,10 @@ def postTaxWarehouse(connection, schema, data):
         data['amount_out'],
         data['amount_out_usd'],
         data['amount_out_eur'],
-        data['tax_amount_usd'],
-        data['tax_amount_eur'],
+        data['profit_amount_usd'],
+        data['profit_amount_eur'],
+        data['fee_amount_usd'],
+        data['fee_amount_eur'],
         data['rest_tx_no'],
         data['rest_amount']
     )
@@ -181,7 +208,7 @@ def postTaxWarehouse(connection, schema, data):
     return result
 
 def postTaxTrade(connection, schema, data):
-    query = sql.SQL("INSERT INTO {}.tax_trades (chain_id,chain,account,address,token,buy_timestamp,buy_price,buy_transaction_hashes,sell_timestamp,sell_price,sell_transaction_hashes,amount) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))   
+    query = sql.SQL("INSERT INTO {}.tax_trades (chain_id,chain,account,address,token,buy_timestamp,buy_price,buy_transaction_hashes,sell_timestamp,sell_price,sell_transaction_hashes,amount,fee_amount_usd,fee_amount_eur) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)").format(sql.Identifier(schema))   
     params = (
         data['chain_id'],
         data['chain'],
@@ -194,7 +221,9 @@ def postTaxTrade(connection, schema, data):
         data['sell_timestamp'],
         data['sell_price'],
         data['sell_transaction_hashes'],
-        data['amount']
+        data['amount'],
+        data['fee_amount_usd'],
+        data['fee_amount_eur']
     )
 
     result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
@@ -250,5 +279,5 @@ def postTaxTrade(connection, schema, data):
     # }
     #res = getTaxTransactionRewards(connection,'stage',['dfi1','df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct'],'DeFiChain',None,'DFI',0)
     #res = getTaxTokens(connection,'stage',['dfi1','df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct'],'DeFiChain',None)
-    # res = getLatestWarehouseDate(connection,'stage','BLUE1','MYACC','Defi',None)
+    # res = getLatestWarehouseDate(connection,'stage','BLUE12','MYACC','Defi',None)
     # print(res)
\ No newline at end of file
